﻿// application tool
(function () {

    Light.action.leancloud = Light.action.leancloud || {};

    Light.action.leancloud.share = {

        //取得全部好友（yxm）
        getAddressBook: function (okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }
            //existUserId: user.id + ",2,3"
            var socialUser = {
                userId: user.id,
                existUserId: user.id + ""
            };
            //findAddressBook
            AV.Cloud.run('findAddressBook', socialUser, {
                success: function (result) {
                    if (okfunc) okfunc(result);
                },
                error: function (error) {
                    //alert("error");
                    if (errfunc) errfunc(error);
                }
            });
        },

        getFriendInfo: function (memberId, okfunc, errfunc) {
            //var memberId = "558ba4e2e4b016384144f2f0";
            var query = new AV.Query("_User");
            query.get(memberId, {
                success: function (follower) {
                    if (okfunc) okfunc(follower);
                },
                error: function (object, error) {
                    if (errfunc) errfunc(error);
                    // The object was not retrieved successfully.
                    // error is a AV.Error with an error code and description.
                }
            });
        },

        //查找好友（yxm）
        getFriends: function (nickName, okfunc, errfunc) {
             var nickName = "%" + nickName + "%";
             var phone = "%" + nickName + "%";
             AV.Query.doCloudQuery('select count(*),* from _User where nickname like ? or phone like ?', [nickName,phone], {
                success: function (result) {
                    //results 是查询返回的结果，AV.Object 列表
                    var results = result.results;
                    var count = result.count;
                    if (okfunc) okfunc(results);
                    //do something with results...
                },
                error: function (error) {
                    //查询失败，查看 error
                    if (errfunc) errfunc(error);
                }
            });
        },

        //关注用户
        follow: function (memberId, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var query = user.followeeQuery();
            query.include('followee');
            query.find().then(function (data) {
                var flg = true;
                for (i = 0; i < data.length; i++) {
                    if (data[i].id == memberId) {
                        flg = false;
                        break;
                    }
                }
                if (flg) {
                    user.follow(memberId).then(function () {
                        if (okfunc) okfunc();
                    }, function (err) {
                        if (errfunc) errfunc(err);
                    })
                } else {
                    if (okfunc) okfunc();
                }

            }, function (error) {
                if (errfunc) errfunc();
            });
        },

        //取消关注用户
        unfollow: function (memberId, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            user.unfollow(memberId).then(function () {
                if (okfunc) okfunc();
            }, function (err) {
                if (errfunc) errfunc(err);
            });
        },

        //取得自己关注的用户
        getFollowee: function (okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var query = user.followeeQuery();
            query.include('followee');
            query.find().then(function (followees) {
                //关注的用户列表followees
                if (okfunc) okfunc(followees);
            }, function (error) {
                if (errfunc) errfunc();
            });
        },

        //取得关注自己的用户（粉丝）
        getFollower: function (okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }
            var query = user.followerQuery();
            query.include('follower');
            query.find().then(function (followers) {
                if (okfunc) okfunc(followers);
            }, function (error) {
                if (errfunc) errfunc();
            });
        },

        //发送私信给用户
        sendPrivateStatus: function (userId, image, data, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var status = new AV.Status(image, data);
            AV.Status.sendPrivateStatus(status, userId).
              then(function (status) {
                  if (okfunc) okfunc(status);
              }, function (err) {
                  if (errfunc) errfunc();
              });
        },

        //取得自己的私信数量
        getPrivateInboxCount: function (okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            AV.Status.countUnreadStatuses(user, "private").then(function (result) {
                if (okfunc) okfunc(result);
            }, function (err) {
                if (errfunc) errfunc();
            });
        },

        //取得自己的私信
        getPrivateInbox: function (sinceId, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var query = AV.Status.inboxQuery(user, "private");
            query.sinceId(sinceId);
            query.find().then(function (statuses) {
                if (okfunc) okfunc(statuses);
            }, function (err) {
                if (errfunc) errfunc();
            });
        },

        //发送给粉丝分享
        sendStatusToFollowersBox: function (boxname, image, data, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var status = new AV.Status(image, data);
            if (!boxname) { boxname = "default"; }
            status.inboxType = boxname;
            AV.Status.sendStatusToFollowers(status).then(function (status) {
                //发布状态成功，返回状态信息
                if (okfunc) okfunc();
            }, function (err) {
                //发布失败
                if (errfunc) errfunc();
            });
        },

        //发送给用户
        sendStatusToUserBox: function (userId, boxname, image, data, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var query = new AV.Query(AV.User);
            query.equalTo("objectId", userId);
            var status = new AV.Status(image, data);
            status.inboxType = boxname;
            status.query = query;
            status.send(status).then(function (status) {
                  if (okfunc) okfunc(status);
              }, function (err) {
                  if (errfunc) errfunc();
              });
        },

        //接受收件箱数量
        getInboxCount: function (boxname, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            AV.Status.countUnreadStatuses(user, boxname).then(function (result) {
                if (okfunc) okfunc(result);
            }, function (err) {
                if (errfunc) errfunc();
            });
        },

        getInbox: function (boxname, sinceId, okfunc, errfunc) {//timeline
            
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var query = AV.Status.inboxQuery(user, boxname);
            query.sinceId(sinceId);
            query.find().then(function (statuses) {
                if (okfunc) okfunc(statuses);
            }, function (error) {
                if (errfunc) errfunc(error);
            });
        },

        //fa件箱
        getStatus: function (okfunc, errfunc) {//timeline
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }
            var query = AV.Status.statusQuery(AV.User.current());
            query.find().then(function (statuses) {
                if (okfunc) okfunc(statuses);
            }, function (error) {
                if (errfunc) errfunc(error);
            })
        },

        checkFollowee: function (userId, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }
            
            var userquery = new AV.Query(AV.User);
            userquery.get(userId, {
                success: function (tuser) {
                    var query = user.followeeQuery();
                    query.include('followee');
                    query.equalTo("followee", tuser);
                    query.find().then(function (followees) {
                        if (okfunc) okfunc(followees);
                    }, function (error) {
                        if (errfunc) errfunc(error);
                    })
                },
                error: function (model, error) {
                    if (errfunc) errfunc(error);
                }
            });
        },

        checkFollower: function (userId, okfunc, errfunc) {
            var user = AV.User.current();
            if (!user) {
                if (errfunc) errfunc();
                return;
            }

            var userquery = new AV.Query(AV.User);
            userquery.get(userId, {
                success: function (tuser) {
                    var query = user.followerQuery();
                    query.include('follower');
                    query.equalTo("follower", tuser);
                    query.find().then(function (followees) {
                        if (okfunc) okfunc(followees);
                    }, function (error) {
                        if (errfunc) errfunc(error);
                    })
                },
                error: function (model, error) {
                    if (errfunc) errfunc(error);
                }
            });
        }
    };

})();
