﻿
Light.db.calorieRecord = {
    name: "calorieRecord",
    store: null,
    scheme: [
        { name: "calorieRecordId", type: "string", default: "" },
        { name: "calorieRecordDate", type: "string", default: "" },
        { name: "calorieInput", type: "float", default: 0 , round: 2},
        { name: "calorieOutput", type: "float", default: 0, round: 2 },
        { name: "calorieOutputMovement", type: "float", default: 0, round: 2 },
        { name: "calorieOutputPedometer", type: "float", default: 0, round: 2 },
        { name: "calorieOutputBase", type: "float", default: 0, round: 2 },
        { name: "uploadFlg", type: "int", default: 0 },
        { name: "timestamp", type: "string", default: "" },
    ],

    init: function () {
        this.store = new DevExpress.data.LocalStore({
            name: "calorieRecord",
            key: "calorieRecordId",
            immediate: true,
            loaded: function () {
                Light.db.log("calorieRecord has been loaded");
            },
        });
    },

    reset: function () {
        Light.db.reset(this.store);
    },
    set: function (date, ci, co, com, cop, cob, donefunc, failfunc) {
        var storeref = this.store;
        var schemeref = this.scheme;
        var getdone = function (items) {
            if (items && items.length > 0) {
                var item = items[0];
                Light.db.updateWithCheck(storeref, schemeref,
                item.calorieRecordId,
                {
                    calorieInput: ci,
                    calorieOutput: co,
                    calorieOutputMovement: com,
                    calorieOutputPedometer: cop,
                    calorieOutputBase: cob,
                    uploadFlg: 0
                },
                donefunc, failfunc);
            } else {
                Light.db.insertWithCheck(storeref, schemeref, {
                    calorieRecordId: Light.tool.newGUID(),
                    calorieRecordDate: date,
                    calorieInput:ci,
                    calorieOutput: co,
                    calorieOutputMovement: com,
                    calorieOutputPedometer: cop,
                    calorieOutputBase: cob,
                    uploadFlg: 0,
                    timestamp: ""
                }, donefunc, failfunc);
            }
        }
        Light.db.getAll(this.store, null, 
            ["calorieRecordDate", date]
        , null, getdone, failfunc);
    },

    getOneDate: function (date, donefunc, failfunc) {
        var getdone = function (items) {
            if (items && items.length > 0) {
                donefunc({
                    input: items[0].calorieInput,
                    output: items[0].calorieOutput,
                    outputMovement: items[0].calorieOutputMovement,
                    outputPedometer: items[0].calorieOutputPedometer,
                    outputBase: items[0].calorieOutputBase
                });
            } else {
                donefunc({input:0, output: 0, outputMovement:0, outputPedometer:0, outputBase:0});
            }
        }
        Light.db.getAll(this.store, null,
            ["calorieRecordDate", date],
            null, getdone, failfunc);
    },
    select: function (condition, donefunc, failfunc) {
        Light.db.getAll(this.store, null, condition, null, donefunc, failfunc);
    },
    selectColumnsSort: function (columns, condition, sort, donefunc, failfunc) {
        Light.db.getAll(this.store, columns, condition, sort, donefunc, failfunc);
    },
    insert: function (values, donefunc, failfunc) {
        Light.db.insertWithCheck(this.store, this.scheme, values, donefunc, failfunc);
    },
    remove: function (id, donefunc, failfunc) {
        Light.db.remove(this.store, id, donefunc, failfunc);
    },
    update: function (id, values, donefunc, failfunc) {
        Light.db.updateWithCheck(this.store, this.scheme, id, values, donefunc, failfunc);
    },
};

Light.db.calorieRecord.init();