(function () {

    Light.network = {
        log: function (text) {
            //console.log(text);
        },

        errorlog: function (text) {
            //console.log(text);
        },


        checkConnection: function () {
            if (navigator.connection) {
                var networkState = navigator.connection.type;

                var states = {};
                states[Connection.UNKNOWN] = 'Unknown connection';
                states[Connection.ETHERNET] = 'Ethernet connection';
                states[Connection.WIFI] = 'WiFi connection';
                states[Connection.CELL_2G] = 'Cell 2G connection';
                states[Connection.CELL_3G] = 'Cell 3G connection';
                states[Connection.CELL_4G] = 'Cell 4G connection';
                states[Connection.CELL] = 'Cell generic connection';
                states[Connection.NONE] = 'No network connection';

                if (networkState == Connection.NONE || networkState == Connection.UNKNOWN) return null;
                return networkState;

            } else {
                //return null;
                return true;
            }
            
        },

        showError: function (type) {
            if (type == "checksum") {
                DevExpress.ui.notify("数据格式错误.", "error", 400);
            } else if (type == "connect") {
                DevExpress.ui.notify("咦？网络不见了，请检查网络连接", "error", 400);
            } else if (type == "acc") {
                DevExpress.ui.notify("当前账号已在另一台手机上使用，本机被强制退出。", "error", 10000);
                Light.action.resetNative();
            } else if (type == "udid") {
                DevExpress.ui.notify("用户身份异常.", "error", 400);
            } else {
                DevExpress.ui.notify("未知错误.", "error", 400);
            }
        },
        getUDID: function (jpushid, deviceinfo, donefunc, failfunc) {
            var thisref = this;
            var action = "/app/member/acc/getUDID";

            var jsonstr = JSON.stringify(deviceinfo);
            this.log("Request:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " [ jpushid:" + jpushid + " deviceinfo:" + jsonstr + "] done ");

            $.ajax({
                type: "POST",
                url: Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action,
                data: {
                    jpushId: jpushid,
                    deviceInfo: jsonstr
                }
            })
            .done(function (msg) {
                thisref.log("Response getUDID done:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " msg:[" + msg + "]");

                var obj = $.parseJSON(msg);
                if (obj.isSuccess == 0) {
                    if (donefunc) donefunc(obj.data.UDID);
                } else if (obj.isSuccess == 1) {
                    if (failfunc) failfunc("logic");
                }

            }).fail(function (requestData, status, e) {
                thisref.errorlog("Response getUDID fail:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " status:[" + status + "]");
                thisref.showError("connect");
                if (failfunc) failfunc("connect");
            });
        },
        /**
        request to server
        **/
        request: function (action, data, donefunc, failfunc, method) {
            var thisref = this;
            var request_type = "GET";
            if (method && method == "post") { request_type = "POST"; }

            var rdata = {
                appVersion: Light.rtdata.VERSION,
                UDID: Light.rtdata.UDID,
                memberAcc: Light.rtdata.memberAcc,
                data: data
            };
            var jsonstr = JSON.stringify(rdata);
            var checksum = "" + CryptoJS.HmacSHA1(jsonstr, Light.config.SECRET_KEY) + "";
            thisref.log("Request:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + "[checksum:" + checksum + "] [data:" + jsonstr + "] done ");

            $.ajax({
                type: request_type,
                //beforeSend: function(request) {
                //    request.setRequestHeader("xxxx", "xxxxx");
                //},
                url: Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action,
                data: { jsondata: jsonstr, checksum: "" + checksum },
                //dataType: "json"
            })
            .done(function (msg) {
                thisref.log("Response done:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " msg:[" + msg + "]");

                if (msg.length < 40) {
                    thisref.showError("checksum");
                    if (failfunc) failfunc("checksum");
                    return;
                }

                var rechecksum = msg.substr(0, 40);
                var rejsonstr = msg.substr(40);
                var checksum = "" + CryptoJS.HmacSHA1(rejsonstr, Light.config.SECRET_KEY) + "";
                //alert("checksum:" + rechecksum + "|" + checksum + " data:[" + rejsonstr + "]");

                if (checksum != rechecksum) {
                    thisref.showError("checksum");
                    if (failfunc) failfunc("checksum");
                    return;
                }

                var obj = $.parseJSON(rejsonstr);
                if (obj.isSuccess == 0) {
                    if (donefunc) donefunc(obj.data);
                } else if (obj.isSuccess == 1) {
                    if (failfunc) failfunc("logic");
                } else if (obj.isSuccess == 9) {
                    thisref.showError("checksum");
                    if (failfunc) failfunc("checksum");
                } else if (obj.isSuccess == 8) {
                    thisref.showError("udid");
                    if (failfunc) failfunc("udid");
                } else if (obj.isSuccess == 7) {
                    thisref.showError("acc");
                    if (failfunc) failfunc("acc");
                } else{
                    thisref.showError("unknown");
                    if (failfunc) failfunc("unknown");
                }

            }).fail(function (requestData, status, e) {
                thisref.errorlog("Response fail:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " status:[" + status + "]");
                thisref.showError("connect");
                if (failfunc) failfunc("connect");
            });
        },
        requestBackground: function (action, data, donefunc, failfunc, method) {
            var thisref = this;
            var request_type = "GET";
            if (method && method == "post") { request_type = "POST"; }

            var rdata = {
                appVersion: Light.rtdata.VERSION,
                UDID: Light.rtdata.UDID,
                memberAcc: Light.rtdata.memberAcc,
                data: data
            };
            var jsonstr = JSON.stringify(rdata);
            var checksum = "" + CryptoJS.HmacSHA1(jsonstr, Light.config.SECRET_KEY) + "";
            thisref.log("Request:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + "[checksum:" + checksum + "] [data:" + jsonstr + "] done ");

            $.ajax({
                type: request_type,
                //beforeSend: function(request) {
                //    request.setRequestHeader("xxxx", "xxxxx");
                //},
                url: Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action,
                data: { jsondata: jsonstr, checksum: "" + checksum },
                //dataType: "json"
            })
            .done(function (msg) {
                thisref.log("Response done:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " msg:[" + msg + "]");

                if (msg.length < 40) {
                    if (failfunc) failfunc();
                    return;
                }

                var rechecksum = msg.substr(0, 40);
                var rejsonstr = msg.substr(40);
                var checksum = "" + CryptoJS.HmacSHA1(rejsonstr, Light.config.SECRET_KEY) + "";
                if (checksum != rechecksum) {
                    if (failfunc) failfunc();
                    return;
                }

                var obj = $.parseJSON(rejsonstr);
                if (obj.isSuccess == 0) {
                    if (donefunc) donefunc(obj.data);
                } else {
                    if (obj.isSuccess == 7) {
                        thisref.showError("acc");
                        if (failfunc) failfunc("acc");
                    } else {
                        if (failfunc) failfunc();
                    }
                }

            }).fail(function (requestData, status, e) {
                thisref.errorlog("Response Background fail:" + Light.config.APP_SERVER_URL + Light.config.REST_ROOT + action + " status:[" + status + "]");
                if (failfunc) failfunc();
            });
        },

        requestResearch: function (action, data, donefunc, failfunc, method) {
            var thisref = this;
            var request_type = "GET";
            if (method && method == "post") { request_type = "POST"; }

            var jsonstr = JSON.stringify(data);

            $.ajax({
                type: request_type,
                url: Light.config.RESEARCH_SERVER_URL + Light.config.RESEARCH_REST_ROOT,
                data: { action:action, jsondata: jsonstr},
                //dataType: "json"
            })
            .done(function (msg) {
                thisref.log("Response done:" + Light.config.RESEARCH_SERVER_URL + Light.config.RESEARCH_REST_ROOT + action + " msg:[" + msg + "]");

                var obj = msg;
                if (obj && obj.result == 0) {
                    if (donefunc) donefunc(obj.data);
                } else {
                    thisref.showError("unknown");
                    if (failfunc) failfunc("unknown");
                }

            }).fail(function (requestData, status, e) {
                thisref.errorlog("Response fail:" + Light.config.RESEARCH_SERVER_URL + Light.config.RESEARCH_REST_ROOT + action + " status:[" + status + "]");
                thisref.showError("connect");
                if (failfunc) failfunc("connect");
            });
        },

    };

})();